@@ -41,6 +41,7 @@
 #include <sys/wait.h>
 #include <unistd.h>
 #include <atomic>
+#include <string>
 #include <vector>
 
 #ifdef __APPLE__
@@ -121,14 +122,14 @@ struct sipp_option {
 
 /* Put each option, its help text, and type in this table. */
 struct sipp_option options_table[] = {
-    {"h", NULL, SIPP_OPTION_HELP, NULL, 0},
-    {"help", NULL, SIPP_OPTION_HELP, NULL, 0},
+    {"h", nullptr, SIPP_OPTION_HELP, nullptr, 0},
+    {"help", nullptr, SIPP_OPTION_HELP, nullptr, 0},
 
-    {"", "Scenario file options:", SIPP_HELP_TEXT_HEADER, NULL, 0},
-    {"sd", "Dumps a default scenario (embedded in the SIPp executable)", SIPP_OPTION_SCENARIO, NULL, 0},
-    {"sf", "Loads an alternate XML scenario file.  To learn more about XML scenario syntax, use the -sd option to dump embedded scenarios. They contain all the necessary help.", SIPP_OPTION_SCENARIO, NULL, 2},
-    {"oocsf", "Load out-of-call scenario.", SIPP_OPTION_OOC_SCENARIO, NULL, 2},
-    {"oocsn", "Load out-of-call scenario.", SIPP_OPTION_OOC_SCENARIO, NULL, 2},
+    {"", "Scenario file options:", SIPP_HELP_TEXT_HEADER, nullptr, 0},
+    {"sd", "Dumps a default scenario (embedded in the SIPp executable)", SIPP_OPTION_SCENARIO, nullptr, 0},
+    {"sf", "Loads an alternate XML scenario file.  To learn more about XML scenario syntax, use the -sd option to dump embedded scenarios. They contain all the necessary help.", SIPP_OPTION_SCENARIO, nullptr, 2},
+    {"oocsf", "Load out-of-call scenario.", SIPP_OPTION_OOC_SCENARIO, nullptr, 2},
+    {"oocsn", "Load out-of-call scenario.", SIPP_OPTION_OOC_SCENARIO, nullptr, 2},
     {
         "sn", "Use a default scenario (embedded in the SIPp executable). If this option is omitted, the Standard SipStone UAC scenario is loaded.\n"
         "Available values in this version:\n\n"
@@ -141,10 +142,10 @@ struct sipp_option options_table[] = {
         "- '3pcc-C-A' : Controller A side (must be started after all other 3pcc scenarios)\n"
         "- '3pcc-C-B' : Controller B side.\n"
         "- '3pcc-A'   : A side.\n"
-        "- '3pcc-B'   : B side.\n", SIPP_OPTION_SCENARIO, NULL, 2
+        "- '3pcc-B'   : B side.\n", SIPP_OPTION_SCENARIO, nullptr, 2
     },
 
-    {"", "IP, port and protocol options:", SIPP_HELP_TEXT_HEADER, NULL, 0},
+    {"", "IP, port and protocol options:", SIPP_HELP_TEXT_HEADER, nullptr, 0},
         {
         "t", "Set the transport mode:\n"
         "- u1: UDP with one socket (default),\n"
@@ -162,31 +163,34 @@ struct sipp_option options_table[] = {
 #endif
         "- c1: u1 + compression (only if compression plugin loaded),\n"
         "- cn: un + compression (only if compression plugin loaded).  This plugin is not provided with SIPp.\n"
-        , SIPP_OPTION_TRANSPORT, NULL, 1
+        , SIPP_OPTION_TRANSPORT, nullptr, 1
     },
     {"i", "Set the local IP address for 'Contact:','Via:', and 'From:' headers. Default is primary host IP address.\n", SIPP_OPTION_IP, local_ip, 1},
     {"p", "Set the local port number.  Default is a random free port chosen by the system.", SIPP_OPTION_INT, &user_port, 1},
     {"bind_local", "Bind socket to local IP address, i.e. the local IP address is used as the source IP address.  If SIPp runs in server mode it will only listen on the local IP address instead of all IP addresses.", SIPP_OPTION_SETFLAG, &bind_local, 1},
+#ifdef SO_BINDTODEVICE
+    {"bind_to_device", "Bind socket to the specified network device. Requires superuser permissions.", SIPP_OPTION_STRING, &bind_to_device_name, 1},
+#endif
     {"ci", "Set the local control IP address", SIPP_OPTION_IP, control_ip, 1},
     {"cp", "Set the local control port number. Default is 8888.", SIPP_OPTION_INT, &control_port, 1},
-    {"max_socket", "Set the max number of sockets to open simultaneously. This option is significant if you use one socket per call. Once this limit is reached, traffic is distributed over the sockets already opened. Default value is 50000", SIPP_OPTION_MAX_SOCKET, NULL, 1},
+    {"max_socket", "Set the max number of sockets to open simultaneously. This option is significant if you use one socket per call. Once this limit is reached, traffic is distributed over the sockets already opened. Default value is 50000", SIPP_OPTION_MAX_SOCKET, nullptr, 1},
     {"max_reconnect", "Set the the maximum number of reconnection.", SIPP_OPTION_INT, &reset_number, 1},
     {"reconnect_close", "Should calls be closed on reconnect?", SIPP_OPTION_BOOL, &reset_close, 1},
     {"reconnect_sleep", "How long (in milliseconds) to sleep between the close and reconnect?", SIPP_OPTION_TIME_MS, &reset_sleep, 1},
-    {"rsa", "Set the remote sending address to host:port for sending the messages.", SIPP_OPTION_RSA, NULL, 1},
+    {"rsa", "Set the remote sending address to host:port for sending the messages.", SIPP_OPTION_RSA, nullptr, 1},
 
 #ifdef USE_TLS
     {"tls_cert", "Set the name for TLS Certificate file. Default is 'cacert.pem'", SIPP_OPTION_STRING, &tls_cert_name, 1},
     {"tls_key", "Set the name for TLS Private Key file. Default is 'cakey.pem'", SIPP_OPTION_STRING, &tls_key_name, 1},
     {"tls_ca", "Set the name for TLS CA file. If not specified, X509 verification is not activated.", SIPP_OPTION_STRING, &tls_ca_name, 1},
     {"tls_crl", "Set the name for Certificate Revocation List file. If not specified, X509 CRL is not activated.", SIPP_OPTION_STRING, &tls_crl_name, 1},
-    {"tls_version", "Set the TLS protocol version to use (1.0, 1.1, 1.2) -- default is autonegotiate", SIPP_OPTION_FLOAT, &tls_version, 1},
+    {"tls_version", "Set the TLS protocol version to use (1.0, 1.1, 1.2, 1.3) -- default is autonegotiate", SIPP_OPTION_FLOAT, &tls_version, 1},
 #else
-    {"tls_cert", NULL, SIPP_OPTION_NEED_SSL, NULL, 1},
-    {"tls_key", NULL, SIPP_OPTION_NEED_SSL, NULL, 1},
-    {"tls_ca", NULL, SIPP_OPTION_NEED_SSL, NULL, 1},
-    {"tls_crl", NULL, SIPP_OPTION_NEED_SSL, NULL, 1},
-    {"tls_version", NULL, SIPP_OPTION_NEED_SSL, NULL, 1},
+    {"tls_cert", nullptr, SIPP_OPTION_NEED_SSL, nullptr, 1},
+    {"tls_key", nullptr, SIPP_OPTION_NEED_SSL, nullptr, 1},
+    {"tls_ca", nullptr, SIPP_OPTION_NEED_SSL, nullptr, 1},
+    {"tls_crl", nullptr, SIPP_OPTION_NEED_SSL, nullptr, 1},
+    {"tls_version", nullptr, SIPP_OPTION_NEED_SSL, nullptr, 1},
 #endif
 
 #ifdef USE_SCTP
@@ -197,38 +201,38 @@ struct sipp_option options_table[] = {
     {"pmtu", "Set path MTU for SCTP", SIPP_OPTION_INT, &pmtu, 1},
     {"gracefulclose", "If true, SCTP association will be closed with SHUTDOWN (default).\n If false, SCTP association will be closed by ABORT.\n", SIPP_OPTION_BOOL, &gracefulclose, 1},
 #else
-    {"multihome", NULL, SIPP_OPTION_NEED_SCTP, NULL, 1},
-    {"heartbeat", NULL, SIPP_OPTION_NEED_SCTP, NULL, 1},
-    {"assocmaxret", NULL, SIPP_OPTION_NEED_SCTP, NULL, 1},
-    {"pathmaxret", NULL, SIPP_OPTION_NEED_SCTP, NULL, 1},
-    {"pmtu", NULL, SIPP_OPTION_NEED_SCTP, NULL, 1},
-    {"gracefulclose", NULL, SIPP_OPTION_NEED_SCTP, NULL, 1},
+    {"multihome", nullptr, SIPP_OPTION_NEED_SCTP, nullptr, 1},
+    {"heartbeat", nullptr, SIPP_OPTION_NEED_SCTP, nullptr, 1},
+    {"assocmaxret", nullptr, SIPP_OPTION_NEED_SCTP, nullptr, 1},
+    {"pathmaxret", nullptr, SIPP_OPTION_NEED_SCTP, nullptr, 1},
+    {"pmtu", nullptr, SIPP_OPTION_NEED_SCTP, nullptr, 1},
+    {"gracefulclose", nullptr, SIPP_OPTION_NEED_SCTP, nullptr, 1},
 #endif
 
 
-   {"", "SIPp overall behavior options:", SIPP_HELP_TEXT_HEADER, NULL, 0},
-   {"v", "Display version and copyright information.", SIPP_OPTION_VERSION, NULL, 0},
+   {"", "SIPp overall behavior options:", SIPP_HELP_TEXT_HEADER, nullptr, 0},
+   {"v", "Display version and copyright information.", SIPP_OPTION_VERSION, nullptr, 0},
    {"bg", "Launch SIPp in background mode.", SIPP_OPTION_SETFLAG, &backgroundMode, 1},
    {"nostdin", "Disable stdin.\n", SIPP_OPTION_SETFLAG, &nostdin, 1},
-   {"plugin", "Load a plugin.", SIPP_OPTION_PLUGIN, NULL, 1},
+   {"plugin", "Load a plugin.", SIPP_OPTION_PLUGIN, nullptr, 1},
    {"sleep", "How long to sleep for at startup. Default unit is seconds.", SIPP_OPTION_TIME_SEC, &sleeptime, 1},
    {"skip_rlimit", "Do not perform rlimit tuning of file descriptor limits.  Default: false.", SIPP_OPTION_SETFLAG, &skip_rlimit, 1},
    {"buff_size", "Set the send and receive buffer size.", SIPP_OPTION_INT, &buff_size, 1},
    {"sendbuffer_warn", "Produce warnings instead of errors on SendBuffer failures.", SIPP_OPTION_BOOL, &sendbuffer_warn, 1},
    {"lost", "Set the number of packets to lose by default (scenario specifications override this value).", SIPP_OPTION_FLOAT, &global_lost, 1},
-   {"key", "keyword value\nSet the generic parameter named \"keyword\" to \"value\".", SIPP_OPTION_KEY, NULL, 1},
-   {"set", "variable value\nSet the global variable parameter named \"variable\" to \"value\".", SIPP_OPTION_VAR, NULL, 3},
+   {"key", "keyword value\nSet the generic parameter named \"keyword\" to \"value\".", SIPP_OPTION_KEY, nullptr, 1},
+   {"set", "variable value\nSet the global variable parameter named \"variable\" to \"value\".", SIPP_OPTION_VAR, nullptr, 3},
    {"tdmmap", "Generate and handle a table of TDM circuits.\n"
     "A circuit must be available for the call to be placed.\n"
-    "Format: -tdmmap {0-3}{99}{5-8}{1-31}", SIPP_OPTION_TDMMAP, NULL, 1},
+    "Format: -tdmmap {0-3}{99}{5-8}{1-31}", SIPP_OPTION_TDMMAP, nullptr, 1},
    {"dynamicStart", "variable value\nSet the start offset of dynamic_id variable",  SIPP_OPTION_INT, &startDynamicId, 1},
    {"dynamicMax",   "variable value\nSet the maximum of dynamic_id variable     ",   SIPP_OPTION_INT, &maxDynamicId,   1},
    {"dynamicStep",  "variable value\nSet the increment of dynamic_id variable",      SIPP_OPTION_INT, &stepDynamicId,  1},
 
 
-    {"", "Call behavior options:", SIPP_HELP_TEXT_HEADER, NULL, 0},
+    {"", "Call behavior options:", SIPP_HELP_TEXT_HEADER, nullptr, 0},
     {"aa", "Enable automatic 200 OK answer for INFO, NOTIFY, OPTIONS and UPDATE.", SIPP_OPTION_SETFLAG, &auto_answer, 1},
-    {"base_cseq", "Start value of [cseq] for each call.", SIPP_OPTION_CSEQ, NULL, 1},
+    {"base_cseq", "Start value of [cseq] for each call.", SIPP_OPTION_CSEQ, nullptr, 1},
     {"cid_str", "Call ID string (default %u-%p@%s).  %u=call_number, %s=ip_address, %p=process_number, %%=% (in any order).", SIPP_OPTION_STRING, &call_id_string, 1},
     {"d", "Controls the length of calls. More precisely, this controls the duration of 'pause' instructions in the scenario, if they do not have a 'milliseconds' section. Default value is 0 and default unit is milliseconds.", SIPP_OPTION_TIME_MS, &duration, 1},
     {"deadcall_wait", "How long the Call-ID and final status of calls should be kept to improve message and error logs (default unit is ms).", SIPP_OPTION_TIME_MS, &deadcall_wait, 1},
@@ -259,17 +263,17 @@ struct sipp_option options_table[] = {
     {"callid_slash_ign", "Don't treat a triple-slash in Call-IDs as indicating an extra SIPp prefix.", SIPP_OPTION_SETFLAG, &callidSlash, 1},
 
 
-    {"", "Injection file options:", SIPP_HELP_TEXT_HEADER, NULL, 0},
+    {"", "Injection file options:", SIPP_HELP_TEXT_HEADER, nullptr, 0},
     {"inf", "Inject values from an external CSV file during calls into the scenarios.\n"
      "First line of this file say whether the data is to be read in sequence (SEQUENTIAL), random (RANDOM), or user (USER) order.\n"
-     "Each line corresponds to one call and has one or more ';' delimited data fields. Those fields can be referred as [field0], [field1], ... in the xml scenario file.  Several CSV files can be used simultaneously (syntax: -inf f1.csv -inf f2.csv ...)", SIPP_OPTION_INPUT_FILE, NULL, 1},
-    {"infindex", "file field\nCreate an index of file using field.  For example -inf ../path/to/users.csv -infindex users.csv 0 creates an index on the first key.", SIPP_OPTION_INDEX_FILE, NULL, 1 },
+     "Each line corresponds to one call and has one or more ';' delimited data fields. Those fields can be referred as [field0], [field1], ... in the xml scenario file.  Several CSV files can be used simultaneously (syntax: -inf f1.csv -inf f2.csv ...)", SIPP_OPTION_INPUT_FILE, nullptr, 1},
+    {"infindex", "file field\nCreate an index of file using field.  For example -inf ../path/to/users.csv -infindex users.csv 0 creates an index on the first key.", SIPP_OPTION_INDEX_FILE, nullptr, 1 },
     {"ip_field", "Set which field from the injection file contains the IP address from which the client will send its messages.\n"
      "If this option is omitted and the '-t ui' option is present, then field 0 is assumed.\n"
      "Use this option together with '-t ui'", SIPP_OPTION_INT, &peripfield, 1},
 
 
-    {"", "RTP behaviour options:", SIPP_HELP_TEXT_HEADER, NULL, 0},
+    {"", "RTP behaviour options:", SIPP_HELP_TEXT_HEADER, nullptr, 0},
     {"mi", "Set the local media IP address (default: local primary host IP address)", SIPP_OPTION_IP, media_ip, 1},
     {"rtp_echo", "Enable RTP echo. RTP/UDP packets received on media port are echoed to their sender.\n"
      "RTP/UDP packets coming on this port + 2 are also echoed to their sender (used for sound and video echo).",
@@ -277,6 +281,7 @@ struct sipp_option options_table[] = {
     {"mb", "Set the RTP echo buffer size (default: 2048).", SIPP_OPTION_INT, &media_bufsize, 1},
     {"min_rtp_port", "Minimum port number for RTP socket range.", SIPP_OPTION_INT, &min_rtp_port, 1},
     {"max_rtp_port", "Maximum port number for RTP socket range.", SIPP_OPTION_INT, &max_rtp_port, 1},
+    {"mp", "Sets -min_rtp_port for backwards compatibility.", SIPP_OPTION_INT, &min_rtp_port, 1},
     {"rtp_payload", "RTP default payload type.", SIPP_OPTION_INT, &rtp_default_payload, 1},
     {"rtp_threadtasks", "RTP number of playback tasks per thread.", SIPP_OPTION_INT, &rtp_tasks_per_thread, 1},
     {"rtp_buffsize", "Set the rtp socket send/receive buffer size.", SIPP_OPTION_INT, &rtp_buffsize, 1},
@@ -287,7 +292,7 @@ struct sipp_option options_table[] = {
     {"audiotolerance", "Audio error tolerance for RTP checks (0.0-1.0) -- default: 1.0", SIPP_OPTION_FLOAT, &audiotolerance, 1},
     {"videotolerance", "Video error tolerance for RTP checks (0.0-1.0) -- default: 1.0", SIPP_OPTION_FLOAT, &videotolerance, 1},
 
-    {"", "Call rate options:", SIPP_HELP_TEXT_HEADER, NULL, 0},
+    {"", "Call rate options:", SIPP_HELP_TEXT_HEADER, nullptr, 0},
     {"r", "Set the call rate (in calls per seconds).  This value can be"
      "changed during test by pressing '+', '_', '*' or '/'. Default is 10.\n"
      "pressing '+' key to increase call rate by 1 * rate_scale,\n"
@@ -309,12 +314,12 @@ struct sipp_option options_table[] = {
     {"no_rate_quit", "If -rate_increase is set, do not quit after the rate reaches -rate_max.", SIPP_OPTION_UNSETFLAG, &rate_quit, 1},
 
     {"l", "Set the maximum number of simultaneous calls. Once this limit is reached, traffic is decreased until the number of open calls goes down. Default:\n"
-     "  (3 * call_duration (s) * rate).", SIPP_OPTION_LIMIT, NULL, 1},
+     "  (3 * call_duration (s) * rate).", SIPP_OPTION_LIMIT, nullptr, 1},
     {"m", "Stop the test and exit when 'calls' calls are processed", SIPP_OPTION_LONG, &stop_after, 1},
-    {"users", "Instead of starting calls at a fixed rate, begin 'users' calls at startup, and keep the number of calls constant.", SIPP_OPTION_USERS, NULL, 1},
+    {"users", "Instead of starting calls at a fixed rate, begin 'users' calls at startup, and keep the number of calls constant.", SIPP_OPTION_USERS, nullptr, 1},
 
 
-    {"", "Retransmission and timeout options:", SIPP_HELP_TEXT_HEADER, NULL, 0},
+    {"", "Retransmission and timeout options:", SIPP_HELP_TEXT_HEADER, nullptr, 0},
     {"recv_timeout", "Global receive timeout. Default unit is milliseconds. If the expected message is not received, the call times out and is aborted.", SIPP_OPTION_TIME_MS_LONG, &defl_recv_timeout, 1},
     {"send_timeout", "Global send timeout. Default unit is milliseconds. If a message is not sent (due to congestion), the call times out and is aborted.", SIPP_OPTION_TIME_MS_LONG, &defl_send_timeout, 1},
     {"timeout", "Global timeout. Default unit is seconds.  If this option is set, SIPp quits after nb units (-timeout 20s quits after 20 seconds).", SIPP_OPTION_TIME_SEC, &global_timeout, 1},
@@ -327,17 +332,17 @@ struct sipp_option options_table[] = {
     {"T2", "Global T2-timer in milli seconds", SIPP_OPTION_TIME_MS, &global_t2, 1},
 
 
-    {"", "Third-party call control options:", SIPP_HELP_TEXT_HEADER, NULL, 0},
+    {"", "Third-party call control options:", SIPP_HELP_TEXT_HEADER, nullptr, 0},
     {"3pcc", "Launch the tool in 3pcc mode (\"Third Party call control\"). The passed IP address depends on the 3PCC role.\n"
      "- When the first twin command is 'sendCmd' then this is the address of the remote twin socket.  SIPp will try to connect to this address:port to send the twin command (This instance must be started after all other 3PCC scenarios).\n"
      "    Example: 3PCC-C-A scenario.\n"
      "- When the first twin command is 'recvCmd' then this is the address of the local twin socket. SIPp will open this address:port to listen for twin command.\n"
-     "    Example: 3PCC-C-B scenario.", SIPP_OPTION_3PCC, NULL, 1},
+     "    Example: 3PCC-C-B scenario.", SIPP_OPTION_3PCC, nullptr, 1},
     {"master","3pcc extended mode: indicates the master number", SIPP_OPTION_3PCC_EXTENDED, &master_name, 1},
     {"slave", "3pcc extended mode: indicates the slave number", SIPP_OPTION_3PCC_EXTENDED, &slave_number, 1},
-    {"slave_cfg", "3pcc extended mode: indicates the file where the master and slave addresses are stored", SIPP_OPTION_SLAVE_CFG, NULL, 1},
+    {"slave_cfg", "3pcc extended mode: indicates the file where the master and slave addresses are stored", SIPP_OPTION_SLAVE_CFG, nullptr, 1},
 
-    {"", "Performance and watchdog options:", SIPP_HELP_TEXT_HEADER, NULL, 0},
+    {"", "Performance and watchdog options:", SIPP_HELP_TEXT_HEADER, nullptr, 0},
     {"timer_resol", "Set the timer resolution. Default unit is milliseconds.  This option has an impact on timers precision."
      "Small values allow more precise scheduling but impacts CPU usage."
      "If the compression is on, the value is set to 50ms. The default value is 10ms.", SIPP_OPTION_TIME_MS, &timer_resolution, 1},
@@ -352,7 +357,7 @@ struct sipp_option options_table[] = {
     {"watchdog_minor_maxtriggers", "How many times the minor watchdog timer can be tripped before the test is terminated.  Default is 120.", SIPP_OPTION_INT, &watchdog_minor_maxtriggers, 1},
 
 
-    {"", "Tracing, logging and statistics options:", SIPP_HELP_TEXT_HEADER, NULL, 0},
+    {"", "Tracing, logging and statistics options:", SIPP_HELP_TEXT_HEADER, nullptr, 0},
     {"f", "Set the statistics report frequency on screen. Default is 1 and default unit is seconds.", SIPP_OPTION_TIME_SEC, &report_freq, 1},
 
     {"trace_stat", "Dumps all statistics in <scenario_name>_<pid>.csv file. Use the '-h stat' option for a detailed description of the statistics file content.", SIPP_OPTION_SETFLAG, &dumpInFile, 1},
@@ -407,7 +412,7 @@ static struct sipp_option *find_option(const char* option) {
 
     /* Allow options to start with '-' or '--' */
     if (option[0] != '-') {
-        return NULL;
+        return nullptr;
     }
     option++;
     if (option[0] == '-') {
@@ -420,7 +425,7 @@ static struct sipp_option *find_option(const char* option) {
         }
     }
 
-    return NULL;
+    return nullptr;
 }
 
 /******************** Recv Poll Processing *********************/
@@ -457,7 +462,7 @@ void timeout_alarm(int /*param*/)
     timeout_exit = true;
 }
 
-/* Send loop & trafic generation*/
+/* Send loop & traffic generation*/
 
 static void traffic_thread(int &rtp_errors, int &echo_errors)
 {
@@ -525,9 +530,9 @@ static void traffic_thread(int &rtp_errors, int &echo_errors)
                 for (int i = pollnfds - 1; i >= 0; --i) {
                     sockets[i]->close();
                     if (sockets[i] == ctrl_socket) {
-                        ctrl_socket = NULL;
+                        ctrl_socket = nullptr;
                     } else if (sockets[i] == stdin_socket) {
-                        stdin_socket = NULL;
+                        stdin_socket = nullptr;
                     }
                 }
 
@@ -569,14 +574,14 @@ static void traffic_thread(int &rtp_errors, int &echo_errors)
          * leads to iterate again on the destroyed (deleted)
          * object. Thus, we have to wait ont step befere actual
          * deletion of the object*/
-        task * last = NULL;
+        task * last = nullptr;
 
         task_list::iterator iter;
         for (iter = running_tasks->begin(); iter != running_tasks->end(); iter++) {
             if (last) {
                 last->run();
                 if (sockets_pending_reset.begin() != sockets_pending_reset.end()) {
-                    last = NULL;
+                    last = nullptr;
                     break;
                 }
             }
@@ -616,7 +621,7 @@ static void rtp_echo_thread(void* param)
     int                   rc;
     sigset_t              mask;
     sigfillset(&mask); /* Mask all allowed signals */
-    rc = pthread_sigmask(SIG_BLOCK, &mask, NULL);
+    rc = pthread_sigmask(SIG_BLOCK, &mask, nullptr);
     if (rc) {
         WARNING("pthread_sigmask returned %d", rc);
         return;
@@ -752,8 +757,12 @@ static char* wrap(const char* in, int offset, int size)
 /* If stdout is a TTY, wrap stdout in a call to PAGER (generally less(1)).
  * Returns a pid_t you'll have to pass to end_pager(). */
 static pid_t begin_pager() {
-    char pager[15] = "/usr/bin/pager";
-    char *argv[2] = {NULL, NULL};
+    std::vector<std::string> pager_options = {
+        "/usr/bin/pager",
+        "/usr/bin/less",
+        "/usr/bin/more"
+    };
+    char *argv[2] = {nullptr, nullptr};
 
     int stdout_fd = fileno(stdout);
     int read_write[2];
@@ -763,17 +772,29 @@ static pid_t begin_pager() {
         return 0;
     }
 
-    /* Get pager first, so we can bail if it's not there */
-    argv[0] = getenv("PAGER");
-    if (!argv[0]) {
-        argv[0] = pager; /* missing PAGER */
-    } else if (!*argv[0]) {
-        return 0; /* blank PAGER */
+    /* Get pager first, so we can bail if it's not there. Prefer env */
+    if (const char *ptr = getenv("PAGER")) {
+        if (!ptr[0]) {
+            return 0; /* blank PAGER */
+        }
+        pager_options.clear();
+        pager_options.push_back(ptr);
     }
 
-    /* Should use euidaccess(3), but requires _GNU_SOURCE */
-    if (access(argv[0], X_OK) < 0) {
-        perror(argv[0]);
+    /* Should use euidaccess(3) instead of access(), but that requires
+     * _GNU_SOURCE */
+    for (std::string &opt : pager_options) {
+        if (access(opt.c_str(), X_OK) == 0) {
+            argv[0] = &opt[0];
+            break;
+        }
+    }
+
+    /* Nothing found? Silently ignore */
+    if (!argv[0]) {
+        if (pager_options.size() == 1) {
+            perror(pager_options[0].c_str()); /* env supplied PAGER not found */
+        }
         return 0;
     }
 
@@ -1019,8 +1040,8 @@ static void print_last_stats()
 
 static void stop_all_traces()
 {
-    message_lfi.fptr = NULL;
-    log_lfi.fptr = NULL;
+    message_lfi.fptr = nullptr;
+    log_lfi.fptr = nullptr;
     dumpInRtt = 0;
     dumpInFile = 0;
 }
@@ -1036,7 +1057,7 @@ static void freeUserVarMap()
 {
     for (int_vt_map::iterator vt_it = userVarMap.begin(); vt_it != userVarMap.end(); vt_it++) {
         vt_it->second->putTable();
-        userVarMap[vt_it->first] = NULL;
+        userVarMap[vt_it->first] = nullptr;
     }
 }
 
@@ -1068,7 +1089,7 @@ static void manage_oversized_file(int signum)
     fflush(f);
     stop_all_traces();
     print_all_responses = 0;
-    error_lfi.fptr = NULL;
+    error_lfi.fptr = nullptr;
 }
 
 static void releaseGlobalAllocations()
@@ -1089,7 +1110,7 @@ void sipp_exit(int rc, int rtp_errors, int echo_errors)
     unsigned long counter_value_success = 0;
 
     /* Some signals may be delivered twice during exit() execution,
-       and we must prevent all this from beeing done twice */
+       and we must prevent all this from being done twice */
 
     {
         static int already_exited = 0;
@@ -1104,17 +1125,17 @@ void sipp_exit(int rc, int rtp_errors, int echo_errors)
     print_errors();
     if (sp) {
         delete sp;
-        sp = NULL;
+        sp = nullptr;
     }
 
     /* Close open files. */
     struct logfile_info** logfile_ptr;
     struct logfile_info* logfiles[] = {
-        &screen_lfi, &calldebug_lfi, &message_lfi, &shortmessage_lfi, &log_lfi, &error_lfi, NULL};
+        &screen_lfi, &calldebug_lfi, &message_lfi, &shortmessage_lfi, &log_lfi, &error_lfi, nullptr};
     for (logfile_ptr = logfiles; *logfile_ptr; ++logfile_ptr) {
         if ((*logfile_ptr)->fptr) {
             fclose((*logfile_ptr)->fptr);
-            (*logfile_ptr)->fptr = NULL;
+            (*logfile_ptr)->fptr = nullptr;
         }
     }
 
@@ -1168,9 +1189,9 @@ static void sighandle_set()
     action_quit.sa_handler = sipp_sighandler;
     action_file_size_exceeded.sa_handler = manage_oversized_file;
 
-    sigaction(SIGTERM, &action_quit, NULL);
-    sigaction(SIGINT, &action_quit, NULL);
-    sigaction(SIGXFSZ, &action_file_size_exceeded, NULL);  // avoid core dump if the max file size is exceeded
+    sigaction(SIGTERM, &action_quit, nullptr);
+    sigaction(SIGINT, &action_quit, nullptr);
+    sigaction(SIGXFSZ, &action_file_size_exceeded, nullptr);  // avoid core dump if the max file size is exceeded
 }
 
 static void set_scenario(const char* name)
@@ -1280,7 +1301,7 @@ static void setup_media_sockets()
 
     /* Resolving local IP */
     if (getaddrinfo(media_ip,
-                    NULL,
+                    nullptr,
                     &hints,
                     &local_addr) != 0) {
         ERROR("Unknown RTP address '%s'.\n"
@@ -1318,7 +1339,6 @@ int main(int argc, char *argv[])
 {
     int                  argi = 0;
     pthread_t pthread2_id = 0, pthread3_id = 0;
-    unsigned int         generic_count = 0;
     bool                 slave_masterSet = false;
     int rtp_errors;
     int echo_errors;
@@ -1326,7 +1346,7 @@ int main(int argc, char *argv[])
     rtp_errors = 0;
     echo_errors = 0;
 
-    generic[0] = NULL;
+    srand(time(nullptr));
 
     /* At least one argument is needed */
     if (argc < 2) {
@@ -1338,24 +1358,24 @@ int main(int argc, char *argv[])
         struct sigaction action_pipe;
         memset(&action_pipe, 0, sizeof(action_pipe));
         action_pipe.sa_handler=SIG_IGN;
-        sigaction(SIGPIPE, &action_pipe, NULL);
+        sigaction(SIGPIPE, &action_pipe, nullptr);
 
         /* The Window Size change Signal is also useless, and causes failures. */
 #ifdef SIGWINCH
-        sigaction(SIGWINCH, &action_pipe, NULL);
+        sigaction(SIGWINCH, &action_pipe, nullptr);
 #endif
 
         /* sig usr1 management */
         struct sigaction action_usr1;
         memset(&action_usr1, 0, sizeof(action_usr1));
         action_usr1.sa_handler = sipp_sigusr1;
-        sigaction(SIGUSR1, &action_usr1, NULL);
+        sigaction(SIGUSR1, &action_usr1, nullptr);
 
         /* sig usr2 management */
         struct sigaction action_usr2;
         memset(&action_usr2, 0, sizeof(action_usr2));
         action_usr2.sa_handler = sipp_sigusr2;
-        sigaction(SIGUSR2, &action_usr2, NULL);
+        sigaction(SIGUSR2, &action_usr2, nullptr);
     }
 
     pid = getpid();
@@ -1367,7 +1387,7 @@ int main(int argc, char *argv[])
     memset(control_ip, 0, sizeof(control_ip));
 
     /* Initialize our global variable structure. */
-    globalVariables = new AllocVariableTable(NULL);
+    globalVariables = new AllocVariableTable(nullptr);
     userVariables = new AllocVariableTable(globalVariables);
 
     /* Command line parsing */
@@ -1411,6 +1431,9 @@ int main(int argc, char *argv[])
 #endif
 #ifdef PCAPPLAY
                        "-PCAP"
+#endif
+#ifdef USE_SHA256
+                       "-SHA256"
 #endif
                        );
 
@@ -1641,11 +1664,7 @@ int main(int argc, char *argv[])
                 REQUIRE_ARG();
                 CHECK_PASS();
 
-                if (generic_count + 1 >= sizeof(generic)/sizeof(generic[0])) {
-                    ERROR("Too many generic parameters %d",generic_count + 1);
-                }
-                generic[generic_count++] = &argv[argi - 1];
-                generic[generic_count] = NULL;
+                generic[argv[argi - 1]] = argv[argi];
                 break;
             case SIPP_OPTION_VAR:
                 REQUIRE_ARG();
@@ -1841,7 +1860,7 @@ int main(int argc, char *argv[])
                             assert(0);
                         }
                     }
-                    token = NULL;
+                    token = nullptr;
                 }
                 break;
             }
@@ -1915,8 +1934,8 @@ int main(int argc, char *argv[])
     }
 
     /* If no scenario was selected, choose the uac one */
-    if (scenario_file == NULL) {
-        assert(main_scenario == NULL);
+    if (scenario_file == nullptr) {
+        assert(main_scenario == nullptr);
         int i = find_scenario("uac");
         set_scenario("uac");
         main_scenario = new scenario(0, i);
@@ -2101,7 +2120,7 @@ int main(int argc, char *argv[])
         new watchdog(watchdog_interval, watchdog_reset, watchdog_major_threshold, watchdog_major_maxtriggers, watchdog_minor_threshold, watchdog_minor_maxtriggers);
     }
 
-    /* Setting the rate and its dependant params (open_calls_allowed) */
+    /* Setting the rate and its dependent params (open_calls_allowed) */
     /* If we are a client, then create the task to open new calls. */
     if (creationMode == MODE_CLIENT) {
         CallGenerationTask::initialize();
@@ -2130,7 +2149,7 @@ int main(int argc, char *argv[])
     }
 
     if (rtp_echo_enabled && media_socket_audio > 0) {
-        if (pthread_create(&pthread2_id, NULL,
+        if (pthread_create(&pthread2_id, nullptr,
                 (void *(*)(void *))rtp_echo_thread, &media_socket_audio) == -1) {
             ERROR_NO("Unable to create RTP echo thread");
         }
@@ -2138,7 +2157,7 @@ int main(int argc, char *argv[])
 
     /* Creating second RTP echo thread for video. */
     if (rtp_echo_enabled && media_socket_video > 0) {
-        if (pthread_create(&pthread3_id, NULL,
+        if (pthread_create(&pthread3_id, nullptr,
                 (void *(*)(void *)) rtp_echo_thread, &media_socket_video) == -1) {
             ERROR_NO("Unable to create video RTP echo thread");
         }
@@ -2149,10 +2168,10 @@ int main(int argc, char *argv[])
     /* Cancel and join other threads. */
     run_echo_thread.store(false, std::memory_order_relaxed);
     if (pthread2_id) {
-        pthread_join(pthread2_id, NULL);
+        pthread_join(pthread2_id, nullptr);
     }
     if (pthread3_id) {
-        pthread_join(pthread3_id, NULL);
+        pthread_join(pthread3_id, nullptr);
     }
 
 #ifdef HAVE_EPOLL
