@@ -45,13 +45,10 @@
 #include "socket.hpp"
 #include "logger.hpp"
 
-/* Older non C++11 gcc (4.6) does not have nullptr */
-#define const_char_nullptr (reinterpret_cast<const char*>(0))
-
 extern bool do_hide;
 
-SIPpSocket *ctrl_socket = NULL;
-SIPpSocket *stdin_socket = NULL;
+SIPpSocket *ctrl_socket = nullptr;
+SIPpSocket *stdin_socket = nullptr;
 
 static int stdin_fileno = -1;
 static int stdin_mode;
@@ -70,7 +67,11 @@ SIPpSocket  *sockets[SIPP_MAXFDS];
 
 int pending_messages = 0;
 
-map<string, SIPpSocket *>     map_perip_fd;
+std::map<std::string, SIPpSocket *>     map_perip_fd;
+
+static void connect_to_peer(
+    char *peer_host, int peer_port, struct sockaddr_storage *peer_sockaddr,
+    char *peer_ip, int peer_ip_size, SIPpSocket **peer_socket);
 
 int gai_getsockaddr(struct sockaddr_storage* ss, const char* host,
                     const char *service, int flags, int family)
@@ -97,7 +98,7 @@ int gai_getsockaddr(struct sockaddr_storage* ss, const char* host,
         snprintf(service, sizeof(service), "%d", port);
         return gai_getsockaddr(ss, host, service, flags, family);
     } else {
-        return gai_getsockaddr(ss, host, const_char_nullptr, flags, family);
+        return gai_getsockaddr(ss, host, nullptr, flags, family);
     }
 }
 
@@ -312,7 +313,7 @@ static bool process_command(char* command)
 }
 
 int command_mode = 0;
-char *command_buffer = NULL;
+char *command_buffer = nullptr;
 
 extern bool sipMsgCheck (const char *P_msg, SIPpSocket *socket);
 
@@ -324,7 +325,7 @@ static const char* get_trimmed_call_id(const char* msg)
      * not recognise the answer to the message sent as being part of an
      * existing call.
      *
-     * Note: [call_id] can be pre-pended with an arbitrary string using
+     * Note: [call_id] can be prepended with an arbitrary string using
      * '///'.
      * Example: Call-ID: ABCDEFGHIJ///[call_id]
      * - it will still be recognized by SIPp as part of the same call.
@@ -340,7 +341,7 @@ static const char* get_trimmed_call_id(const char* msg)
 static char* get_inet_address(const struct sockaddr_storage* addr, char* dst, int len)
 {
     if (getnameinfo(_RCAST(struct sockaddr*, addr), socklen_from_addr(addr),
-                    dst, len, NULL, 0, NI_NUMERICHOST) != 0) {
+                    dst, len, nullptr, 0, NI_NUMERICHOST) != 0) {
         snprintf(dst, len, "addr not supported");
     }
     return dst;
@@ -501,7 +502,7 @@ void setup_ctrl_socket()
 
     memset(&ctl_sa, 0, sizeof(struct sockaddr_storage));
     if (control_ip[0]) {
-        if (gai_getsockaddr(&ctl_sa, control_ip, const_char_nullptr,
+        if (gai_getsockaddr(&ctl_sa, control_ip, nullptr,
                             AI_PASSIVE, AF_UNSPEC) != 0) {
             ERROR("Unknown control address '%s'.\n"
                   "Use 'sipp -h' for details", control_ip);
@@ -533,9 +534,6 @@ void setup_ctrl_socket()
     }
 
     ctrl_socket = new SIPpSocket(0, T_UDP, sock, 0);
-    if (!ctrl_socket) {
-        ERROR_NO("Could not setup control socket!");
-    }
 }
 
 void reset_stdin()
@@ -551,9 +549,6 @@ void setup_stdin_socket()
     fcntl(stdin_fileno, F_SETFL, stdin_mode | O_NONBLOCK);
 
     stdin_socket = new SIPpSocket(0, T_TCP, stdin_fileno, 0);
-    if (!stdin_socket) {
-        ERROR_NO("Could not setup keyboard (stdin) socket!");
-    }
 }
 
 #define SIPP_ENDL "\r\n"
@@ -564,7 +559,7 @@ void handle_stdin_socket()
 
     if (feof(stdin)) {
         stdin_socket->close();
-        stdin_socket = NULL;
+        stdin_socket = nullptr;
         return;
     }
 
@@ -622,7 +617,7 @@ void handle_stdin_socket()
     if (chars == 0) {
         /* We did not read any characters, even though we should have. */
         stdin_socket->close();
-        stdin_socket = NULL;
+        stdin_socket = nullptr;
     }
 }
 
@@ -780,7 +775,7 @@ int SIPpSocket::check_for_message()
         if (socketbuf->offset + len + content_length < socketbuf->len) {
             return len + content_length + 1;
         }
-        if (socketbuf->next == NULL) {
+        if (socketbuf->next == nullptr) {
             /* There is no buffer to merge, so we fail. */
             return 0;
         }
@@ -862,7 +857,7 @@ int SIPpSocket::empty()
     if (!buffer) {
         ERROR("Could not allocate memory for read!");
     }
-    socketbuf = alloc_socketbuf(buffer, readsize, NO_COPY, NULL);
+    socketbuf = alloc_socketbuf(buffer, readsize, NO_COPY, nullptr);
 
     switch(ss_transport) {
     case T_TCP:
@@ -934,7 +929,7 @@ void SIPpSocket::invalidate()
     /* In some error conditions, the socket FD has already been closed - if it hasn't, do so now. */
     if (ss_fd != -1) {
 #ifdef HAVE_EPOLL
-        int rc = epoll_ctl(epollfd, EPOLL_CTL_DEL, ss_fd, NULL);
+        int rc = epoll_ctl(epollfd, EPOLL_CTL_DEL, ss_fd, nullptr);
         if (rc == -1) {
             WARNING_NO("Failed to delete FD from epoll");
         }
@@ -995,7 +990,7 @@ void SIPpSocket::invalidate()
         sockets[pollidx] = sockets[pollnfds];
         sockets[pollidx]->ss_pollidx = pollidx;
     }
-    sockets[pollnfds] = NULL;
+    sockets[pollnfds] = nullptr;
 
     if (ss_msglen) {
         pending_messages--;
@@ -1130,9 +1125,6 @@ void process_message(SIPpSocket *socket, char *msg, ssize_t msg_size, struct soc
             // Adding a new OUTGOING call !
             main_scenario->stats->computeStat(CStat::E_CREATE_OUTGOING_CALL);
             call *new_ptr = new call(call_id, local_ip_is_ipv6, 0, use_remote_sending_addr ? &remote_sending_sockaddr : &remote_sockaddr);
-            if (!new_ptr) {
-                ERROR("Out of memory allocating a call!");
-            }
 
             outbound_congestion = false;
             if ((socket != main_socket) &&
@@ -1170,9 +1162,6 @@ void process_message(SIPpSocket *socket, char *msg, ssize_t msg_size, struct soc
             // Adding a new INCOMING call !
             main_scenario->stats->computeStat(CStat::E_CREATE_INCOMING_CALL);
             listener_ptr = new call(call_id, socket, use_remote_sending_addr ? &remote_sending_sockaddr : src);
-            if (!listener_ptr) {
-                ERROR("Out of memory allocating a call!");
-            }
         } else { // mode != from SERVER and 3PCC Controller B
             // This is a message that is not relating to any known call
             if (ooc_scenario) {
@@ -1188,9 +1177,6 @@ void process_message(SIPpSocket *socket, char *msg, ssize_t msg_size, struct soc
                     free(msg_start);
                     /* This should have the real address that the message came from. */
                     call *call_ptr = new call(ooc_scenario, socket, use_remote_sending_addr ? &remote_sending_sockaddr : src, call_id, 0 /* no user. */, socket->ss_ipv6, true, false);
-                    if (!call_ptr) {
-                        ERROR("Out of memory allocating a call!");
-                    }
                     CStat::globalStat(CStat::E_AUTO_ANSWERED);
                     call_ptr->process_incoming(msg, src);
                 } else {
@@ -1210,9 +1196,6 @@ void process_message(SIPpSocket *socket, char *msg, ssize_t msg_size, struct soc
                     aa_scenario->stats->computeStat(CStat::E_CREATE_INCOMING_CALL);
                     /* This should have the real address that the message came from. */
                     call *call_ptr = new call(aa_scenario, socket, use_remote_sending_addr ? &remote_sending_sockaddr : src, call_id, 0 /* no user. */, socket->ss_ipv6, true, false);
-                    if (!call_ptr) {
-                        ERROR("Out of memory allocating a call!");
-                    }
                     CStat::globalStat(CStat::E_AUTO_ANSWERED);
                     call_ptr->process_incoming(msg, src);
                 } else {
@@ -1247,26 +1230,26 @@ SIPpSocket::SIPpSocket(bool use_ipv6, int transport, int fd, int accepting):
     ss_control(false),
     ss_fd(fd),
     ss_bind_port(0),
-    ss_comp_state(NULL),
+    ss_comp_state(nullptr),
     ss_changed_dest(false),
     ss_congested(false),
     ss_invalid(false),
-    ss_in(NULL),
-    ss_out(NULL),
-    ss_out_tail(NULL),
+    ss_in(nullptr),
+    ss_out(nullptr),
+    ss_out_tail(nullptr),
     ss_msglen(0)
 {
     /* Initialize all sockets with our destination address. */
     memcpy(&ss_dest, &remote_sockaddr, sizeof(ss_dest));
 
 #if defined(USE_OPENSSL) || defined(USE_WOLFSSL)
-    ss_ssl = NULL;
+    ss_ssl = nullptr;
 
     if (transport == T_TLS) {
         int flags = fcntl(fd, F_GETFL, 0);
         fcntl(fd, F_SETFL, flags | O_NONBLOCK);
 
-        if ((ss_bio = BIO_new_socket(fd, BIO_NOCLOSE)) == NULL) {
+        if ((ss_bio = BIO_new_socket(fd, BIO_NOCLOSE)) == nullptr) {
             ERROR("Unable to create BIO object:Problem with BIO_new_socket()");
         }
 
@@ -1353,7 +1336,7 @@ SIPpSocket *new_sipp_socket(bool use_ipv6, int transport) {
 }
 
 SIPpSocket* SIPpSocket::new_sipp_call_socket(bool use_ipv6, int transport, bool *existing) {
-    SIPpSocket *sock = NULL;
+    SIPpSocket *sock = nullptr;
     static int next_socket;
     if (pollnfds >= max_multi_socket) {  // we must take the main socket into account
         /* Find an existing socket that matches transport and ipv6 parameters. */
@@ -1419,10 +1402,6 @@ SIPpSocket* SIPpSocket::accept() {
 #endif
 
     ret = new SIPpSocket(ss_ipv6, ss_transport, fd, 1);
-    if (!ret) {
-        ::close(fd);
-        ERROR_NO("Could not allocate new socket!");
-    }
 
     /* We should connect back to the address which connected to us if we
      * experience a TCP failure. */
@@ -1608,10 +1587,10 @@ int SIPpSocket::reconnect()
 
     if (ss_invalid) {
 #if defined(USE_OPENSSL) || defined(USE_WOLFSSL)
-        ss_ssl = NULL;
+        ss_ssl = nullptr;
 
         if (transport == T_TLS) {
-            if ((ss_bio = BIO_new_socket(ss_fd, BIO_NOCLOSE)) == NULL) {
+            if ((ss_bio = BIO_new_socket(ss_fd, BIO_NOCLOSE)) == nullptr) {
                 ERROR("Unable to create BIO object:Problem with BIO_new_socket()");
             }
 
@@ -1647,6 +1626,16 @@ int SIPpSocket::reconnect()
     return connect();
 }
 
+#ifdef SO_BINDTODEVICE
+int SIPpSocket::bind_to_device(const char* device_name) {
+    if (setsockopt(this->ss_fd, SOL_SOCKET, SO_BINDTODEVICE,
+                   device_name, strlen(device_name)) == -1) {
+        ERROR_NO("setsockopt(SO_BINDTODEVICE) failed");
+    }
+    return 0;
+}
+#endif
+
 
 /*************************** I/O functions ***************************/
 
@@ -1674,7 +1663,7 @@ struct socketbuf *alloc_socketbuf(char *buffer, size_t size, int copy, struct so
     if (dest) {
         memcpy(&socketbuf->addr, dest, sizeof(*dest));
     }
-    socketbuf->next = NULL;
+    socketbuf->next = nullptr;
 
     return socketbuf;
 }
@@ -1944,7 +1933,7 @@ int SIPpSocket::read_error(int ret)
             } else {
                 /* The socket was closed "cleanly", but we may have calls that need to
                  * be destroyed.  Also, if these calls are not complete, and attempt to
-                 * send again we may "ressurect" the socket by reconnecting it.*/
+                 * send again we may "resurrect" the socket by reconnecting it.*/
                 invalidate();
                 if (reset_close) {
                     close_calls();
@@ -2016,7 +2005,7 @@ static int send_nowait_tls(SSL* ssl, const void* msg, int len, int /*flags*/)
     if ((fd = SSL_get_fd(ssl)) == -1) {
         return -1;
     }
-    fd_flags = fcntl(fd, F_GETFL, NULL);
+    fd_flags = fcntl(fd, F_GETFL, nullptr);
     initial_fd_flags = fd_flags;
     fd_flags |= O_NONBLOCK;
     fcntl(fd, F_SETFL, fd_flags);
@@ -2046,7 +2035,7 @@ static int send_nowait(int s, const void* msg, int len, int flags)
 #if defined(MSG_DONTWAIT) && !defined(__SUNOS)
     return send(s, msg, len, flags | MSG_DONTWAIT);
 #else
-    int fd_flags = fcntl(s, F_GETFL , NULL);
+    int fd_flags = fcntl(s, F_GETFL , nullptr);
     int initial_fd_flags;
     int rc;
 
@@ -2074,7 +2063,7 @@ int send_sctp_nowait(int s, const void *msg, int len, int flags)
 #if defined(MSG_DONTWAIT) && !defined(__SUNOS)
     return sctp_send(s, msg, len, &sinfo, flags | MSG_DONTWAIT);
 #else
-    int fd_flags = fcntl(s, F_GETFL, NULL);
+    int fd_flags = fcntl(s, F_GETFL, nullptr);
     int initial_fd_flags;
     int rc;
 
@@ -2303,7 +2292,7 @@ void SIPpSocket::close_calls()
 {
     owner_list *owners = get_owners_for_socket(this);
     owner_list::iterator owner_it;
-    socketowner *owner_ptr = NULL;
+    socketowner *owner_ptr = nullptr;
 
     for (owner_it = owners->begin(); owner_it != owners->end(); owner_it++) {
         owner_ptr = *owner_it;
@@ -2335,10 +2324,34 @@ int open_connections()
         /* Resolving the remote IP */
         {
             fprintf(stderr, "Resolving remote host '%s'... ", remote_host);
+            struct addrinfo   hints;
+
+            memset((char*)&hints, 0, sizeof(hints));
+            hints.ai_flags  = AI_PASSIVE;
+            hints.ai_family = AF_UNSPEC;
+
+#ifdef USE_LOCAL_IP_HINTS
+            struct addrinfo * local_addr;
+            int ret;
+            if (strlen(local_ip)) {
+                if ((ret = getaddrinfo(local_ip, nullptr, &hints, &local_addr)) != 0) {
+                    ERROR("Can't get local IP address in getaddrinfo, "
+                            "local_ip='%s', ret=%d", local_ip, ret);
+                }
+
+                /* Use local address hints when getting the remote */
+                if (local_addr->ai_addr->sa_family == AF_INET6) {
+                    local_ip_is_ipv6 = true;
+                    hints.ai_family = AF_INET6;
+                } else {
+                    hints.ai_family = AF_INET;
+                }
+            }
+#endif
 
             /* FIXME: add DNS SRV support using liburli? */
             if (gai_getsockaddr(&remote_sockaddr, remote_host, remote_port,
-                                AI_PASSIVE, AF_UNSPEC) != 0) {
+                                hints.ai_flags, hints.ai_family) != 0) {
                 ERROR("Unknown remote host '%s'.\n"
                       "Use 'sipp -h' for details", remote_host);
             }
@@ -2384,7 +2397,7 @@ int open_connections()
             }
 
             /* Resolving local IP */
-            if ((ret = getaddrinfo(local_ip, NULL, &hints, &local_addr)) != 0) {
+            if ((ret = getaddrinfo(local_ip, nullptr, &hints, &local_addr)) != 0) {
               switch (ret) {
 #ifdef EAI_ADDRFAMILY
                 case EAI_ADDRFAMILY:
@@ -2437,12 +2450,19 @@ int open_connections()
     }
 
     /* Creating and binding the local socket */
-    if ((main_socket = new_sipp_socket(local_ip_is_ipv6, transport)) == NULL) {
+    if ((main_socket = new_sipp_socket(local_ip_is_ipv6, transport)) == nullptr) {
         ERROR_NO("Unable to get the local socket");
     }
 
     sipp_customize_socket(main_socket);
 
+#ifdef SO_BINDTODEVICE
+    /* Bind to the device if any. */
+    if (bind_to_device_name) {
+        main_socket->bind_to_device(bind_to_device_name);
+    }
+#endif
+
     /* Trying to bind local port */
     char peripaddr[256];
     if (!user_port) {
@@ -2459,13 +2479,13 @@ int open_connections()
                     // For the socket per IP mode, bind the main socket to the
                     // first IP address specified in the inject file.
                     inFiles[ip_file]->getField(0, peripfield, peripaddr, sizeof(peripaddr));
-                    if (gai_getsockaddr(&local_sockaddr, peripaddr, const_char_nullptr,
+                    if (gai_getsockaddr(&local_sockaddr, peripaddr, nullptr,
                                         AI_PASSIVE, AF_UNSPEC) != 0) {
                         ERROR("Unknown host '%s'.\n"
                               "Use 'sipp -h' for details", peripaddr);
                     }
                 } else {
-                    if (gai_getsockaddr(&local_sockaddr, local_ip, const_char_nullptr,
+                    if (gai_getsockaddr(&local_sockaddr, local_ip, nullptr,
                                         AI_PASSIVE, AF_UNSPEC) != 0) {
                         ERROR("Unknown host '%s'.\n"
                               "Use 'sipp -h' for details", peripaddr);
@@ -2490,13 +2510,13 @@ int open_connections()
                 // For the socket per IP mode, bind the main socket to the
                 // first IP address specified in the inject file.
                 inFiles[ip_file]->getField(0, peripfield, peripaddr, sizeof(peripaddr));
-                if (gai_getsockaddr(&local_sockaddr, peripaddr, const_char_nullptr,
+                if (gai_getsockaddr(&local_sockaddr, peripaddr, nullptr,
                                     AI_PASSIVE, AF_UNSPEC) != 0) {
                     ERROR("Unknown host '%s'.\n"
                           "Use 'sipp -h' for details", peripaddr);
                 }
             } else {
-                if (gai_getsockaddr(&local_sockaddr, local_ip, const_char_nullptr,
+                if (gai_getsockaddr(&local_sockaddr, local_ip, nullptr,
                                     AI_PASSIVE, AF_UNSPEC) != 0) {
                     ERROR("Unknown host '%s'.\n"
                           "Use 'sipp -h' for details", peripaddr);
@@ -2525,8 +2545,7 @@ int open_connections()
         unsigned int lines = inFiles[ip_file]->numLines();
         for (unsigned int i = 0; i < lines; i++) {
             inFiles[ip_file]->getField(i, peripfield, peripaddr, sizeof(peripaddr));
-            map<string, SIPpSocket *>::iterator j;
-            j = map_perip_fd.find(peripaddr);
+            auto j = map_perip_fd.find(peripaddr);
 
             if (j == map_perip_fd.end()) {
                 if (gai_getsockaddr(&server_sockaddr, peripaddr, local_port,
@@ -2537,12 +2556,12 @@ int open_connections()
 
                 bool is_ipv6 = (server_sockaddr.ss_family == AF_INET6);
 
-                if ((sock = new_sipp_socket(is_ipv6, transport)) == NULL) {
+                if ((sock = new_sipp_socket(is_ipv6, transport)) == nullptr) {
                     ERROR_NO("Unable to get server socket");
                 }
 
                 sipp_customize_socket(sock);
-                if (sipp_bind_socket(sock, &server_sockaddr, NULL)) {
+                if (sipp_bind_socket(sock, &server_sockaddr, nullptr)) {
                     ERROR_NO("Unable to bind server socket");
                 }
 
@@ -2553,7 +2572,7 @@ int open_connections()
 
     if ((!multisocket) && (transport == T_TCP || transport == T_TLS || transport == T_SCTP) &&
             (sendMode != MODE_SERVER)) {
-        if ((tcp_multiplex = new_sipp_socket(local_ip_is_ipv6, transport)) == NULL) {
+        if ((tcp_multiplex = new_sipp_socket(local_ip_is_ipv6, transport)) == nullptr) {
             ERROR_NO("Unable to get a TCP socket");
         }
 
@@ -2570,7 +2589,7 @@ int open_connections()
         sipp_customize_socket(tcp_multiplex);
 
         /* This fixes local_port keyword value when transport are TCP|TLS and it's defined by user with "-p" */
-        if (sipp_bind_socket(tcp_multiplex, &local_sockaddr, NULL)) {
+        if (sipp_bind_socket(tcp_multiplex, &local_sockaddr, nullptr)) {
             ERROR_NO("Unable to bind TCP socket");
         }
 
@@ -2578,7 +2597,7 @@ int open_connections()
             if (reset_number > 0) {
                 WARNING("Failed to reconnect");
                 main_socket->close();
-                main_socket = NULL;
+                main_socket = nullptr;
                 reset_number--;
                 return 1;
             } else {
@@ -2605,7 +2624,7 @@ int open_connections()
     /* Trying to connect to Twin Sipp in 3PCC mode */
     if (twinSippMode) {
         if (thirdPartyMode == MODE_3PCC_CONTROLLER_A || thirdPartyMode == MODE_3PCC_A_PASSIVE) {
-            connect_to_peer(twinSippHost, twinSippPort, &twinSipp_sockaddr, twinSippIp, &twinSippSocket);
+            connect_to_peer(twinSippHost, twinSippPort, &twinSipp_sockaddr, twinSippIp, sizeof(twinSippIp), &twinSippSocket);
         } else if (thirdPartyMode == MODE_3PCC_CONTROLLER_B) {
             connect_local_twin_socket(twinSippHost);
         } else {
@@ -2632,7 +2651,9 @@ int open_connections()
 }
 
 
-void connect_to_peer(char *peer_host, int peer_port, struct sockaddr_storage *peer_sockaddr, char *peer_ip, SIPpSocket **peer_socket)
+static void connect_to_peer(
+    char *peer_host, int peer_port, struct sockaddr_storage *peer_sockaddr,
+    char *peer_ip, int peer_ip_size, SIPpSocket **peer_socket)
 {
     /* Resolving the  peer IP */
     printf("Resolving peer address : %s...\n", peer_host);
@@ -2649,9 +2670,9 @@ void connect_to_peer(char *peer_host, int peer_port, struct sockaddr_storage *pe
         is_ipv6 = true;
     }
 
-    get_inet_address(peer_sockaddr, peer_ip, sizeof(peer_ip));
+    get_inet_address(peer_sockaddr, peer_ip, peer_ip_size);
 
-    if ((*peer_socket = new_sipp_socket(is_ipv6, T_TCP)) == NULL) {
+    if ((*peer_socket = new_sipp_socket(is_ipv6, T_TCP)) == nullptr) {
         ERROR_NO("Unable to get a twin sipp TCP socket");
     }
 
@@ -2682,7 +2703,7 @@ SIPpSocket **get_peer_socket(char * peer) {
     } else {
         ERROR("get_peer_socket: Peer %s not found", peer);
     }
-    return NULL;
+    return nullptr;
 }
 
 char * get_peer_addr(char * peer)
@@ -2696,7 +2717,7 @@ char * get_peer_addr(char * peer)
     } else {
         ERROR("get_peer_addr: Peer %s not found", peer);
     }
-    return NULL;
+    return nullptr;
 }
 
 bool is_a_peer_socket(SIPpSocket *peer_socket)
@@ -2729,7 +2750,7 @@ void connect_local_twin_socket(char * twinSippHost)
 
     get_inet_address(&twinSipp_sockaddr, twinSippIp, sizeof(twinSippIp));
 
-    if ((localTwinSippSocket = new_sipp_socket(is_ipv6, T_TCP)) == NULL) {
+    if ((localTwinSippSocket = new_sipp_socket(is_ipv6, T_TCP)) == nullptr) {
         ERROR_NO("Unable to get a listener TCP socket ");
     }
 
@@ -2755,7 +2776,7 @@ void close_peer_sockets()
          ++peer_it) {
         T_peer_infos infos = peer_it->second;
         infos.peer_socket->close();
-        infos.peer_socket = NULL;
+        infos.peer_socket = nullptr;
         peers[std::string(peer_it->first)] = infos;
     }
 
@@ -2766,7 +2787,7 @@ void close_local_sockets()
 {
     for (int i = 0; i< local_nb; i++) {
         local_sockets[i]->close();
-        local_sockets[i] = NULL;
+        local_sockets[i] = nullptr;
     }
 }
 
@@ -2777,7 +2798,7 @@ void connect_to_all_peers()
     for (peer_it = peers.begin(); peer_it != peers.end(); peer_it++) {
         infos = peer_it->second;
         get_host_and_port(infos.peer_host, infos.peer_host, &infos.peer_port);
-        connect_to_peer(infos.peer_host, infos.peer_port, &(infos.peer_sockaddr), infos.peer_ip, &(infos.peer_socket));
+        connect_to_peer(infos.peer_host, infos.peer_port, &(infos.peer_sockaddr), infos.peer_ip, sizeof(infos.peer_ip), &(infos.peer_socket));
         peer_sockets[infos.peer_socket] = peer_it->first;
         peers[std::string(peer_it->first)] = infos;
     }
@@ -3051,7 +3072,7 @@ bool sipMsgCheck (const char *P_msg, SIPpSocket *socket)
             is_a_peer_socket(socket) || is_a_local_socket(socket))
         return true;
 
-    if (strstr(P_msg, C_sipHeader) !=  NULL) {
+    if (strstr(P_msg, C_sipHeader) !=  nullptr) {
         return true;
     }
 
